from typing import Final

CM_AS_INCH: Final[float] = 1.0 / 2.54
FIGURE_HEIGHT_CURTAIN: Final[float] = 8.0 * CM_AS_INCH
FIGURE_WIDTH_CURTAIN: Final[float] = FIGURE_HEIGHT_CURTAIN * 3.0
FIGURE_HEIGHT_SWATH: Final[float] = FIGURE_HEIGHT_CURTAIN * 0.5
FIGURE_WIDTH_SWATH: Final[float] = FIGURE_WIDTH_CURTAIN
FIGURE_HEIGHT_PROFILE: Final[float] = FIGURE_HEIGHT_CURTAIN
FIGURE_WIDTH_PROFILE: Final[float] = FIGURE_HEIGHT_CURTAIN * 0.75
FIGURE_MAP_HEIGHT: Final[float] = FIGURE_HEIGHT_CURTAIN
FIGURE_MAP_WIDTH: Final[float] = FIGURE_HEIGHT_CURTAIN
FIGURE_HEIGHT_LINE: Final[float] = FIGURE_HEIGHT_CURTAIN * 0.5
FIGURE_WIDTH_LINE: Final[float] = FIGURE_WIDTH_CURTAIN
